package de.mdelab.mlsdm.interpreter.patternMatcher.patternPartBased;

import java.util.Collection;
import java.util.Map;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;

import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlstorypatterns.AbstractStoryPatternLink;
import de.mdelab.mlstorypatterns.AbstractStoryPatternObject;
import de.mdelab.mlstorypatterns.ExpressionLink;
import de.mdelab.mlstorypatterns.MatchTypeEnum;
import de.mdelab.mlstorypatterns.ModifierEnum;
import de.mdelab.sdm.interpreter.core.SDMException;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.ECheckResult;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.EMatchType;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.MatchState;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.PatternPart;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.PatternPartBasedMatcher;
import de.mdelab.sdm.interpreter.core.variables.Variable;

/**
 * 
 * @author Stephan Hildebrandt
 * 
 */
public class MLSDMExpressionLinkPatternPart extends MLSDMPatternPart<AbstractStoryPatternObject, ExpressionLink>
{

	public MLSDMExpressionLinkPatternPart(
			final PatternPartBasedMatcher<?, ?, ?, ?, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, ?, MLExpression> patternMatcher,
			final ExpressionLink storyPatternLink)
	{
		super(patternMatcher, storyPatternLink, new AbstractStoryPatternObject[]
		{
				storyPatternLink.getSource(), storyPatternLink.getTarget()
		});

		/*
		 * expression links cannot be created
		 */
		assert storyPatternLink.getModifier() != ModifierEnum.CREATE & storyPatternLink.getSource().getModifier() != ModifierEnum.CREATE
				&& storyPatternLink.getTarget().getModifier() != ModifierEnum.CREATE;

		/*
		 * expression links cannot be destroyed
		 */
		assert storyPatternLink.getModifier() != ModifierEnum.DESTROY & storyPatternLink.getSource().getModifier() != ModifierEnum.DESTROY
				&& storyPatternLink.getTarget().getModifier() != ModifierEnum.DESTROY;

		/*
		 * If one of the link ends is optional, the links must be optional as
		 * well.
		 */
		assert storyPatternLink.getSource().getMatchType() != MatchTypeEnum.OPTIONAL
				|| storyPatternLink.getMatchType() == MatchTypeEnum.OPTIONAL;
		assert storyPatternLink.getTarget().getMatchType() != MatchTypeEnum.OPTIONAL
				|| storyPatternLink.getMatchType() == MatchTypeEnum.OPTIONAL;

		assert storyPatternLink.getExpression() != null;
	}

	@Override
	protected void doCreateLink()
	{
		// Do nothing
	}

	@Override
	protected void doDestroyLink(final Map<AbstractStoryPatternObject, Object> deletedObjects)
	{
		// Do nothing
	}

	@SuppressWarnings("unchecked")
	@Override
	public ECheckResult check() throws SDMException
	{
		if (this.patternMatcher.isBound(this.storyPatternLink.getSource())
				&& this.patternMatcher.isBound(this.storyPatternLink.getTarget()))
		{
			final AbstractStoryPatternObject sourceSpo = this.storyPatternLink.getSource();
			final AbstractStoryPatternObject targetSpo = this.storyPatternLink.getTarget();

			final EObject sourceInstanceObject = (EObject) this.patternMatcher.getInstanceObject(sourceSpo);
			final Object targetInstanceObject = this.patternMatcher.getInstanceObject(targetSpo);

			/*
			 * Evaluate the expression
			 */
			final Variable<EClassifier> result = this.patternMatcher.getExpressionInterpreterManager().evaluateExpression(
					this.storyPatternLink.getExpression(), sourceSpo.getType(), sourceInstanceObject,
					this.patternMatcher.getVariablesScope());

			if (result.getValue() instanceof Collection<?>)
			{
				if (((Collection<Object>) result.getValue()).contains(targetInstanceObject))
				{
					this.patternMatcher.getNotificationEmitter().linkCheckSuccessful(sourceSpo, sourceInstanceObject,
							this.storyPatternLink, targetSpo, targetInstanceObject, this.patternMatcher.getVariablesScope(),
							this.patternMatcher);

					return ECheckResult.OK;
				}
				else
				{
					this.patternMatcher.getNotificationEmitter().linkCheckFailed(sourceSpo, sourceInstanceObject, this.storyPatternLink,
							targetSpo, targetInstanceObject, this.patternMatcher.getVariablesScope(), this.patternMatcher);

					return ECheckResult.FAIL;
				}
			}
			else
			{
				if (result.getValue() == targetInstanceObject)
				{
					this.patternMatcher.getNotificationEmitter().linkCheckSuccessful(sourceSpo, sourceInstanceObject,
							this.storyPatternLink, targetSpo, targetInstanceObject, this.patternMatcher.getVariablesScope(),
							this.patternMatcher);

					return ECheckResult.OK;
				}
				else
				{
					this.patternMatcher.getNotificationEmitter().linkCheckFailed(sourceSpo, sourceInstanceObject, this.storyPatternLink,
							targetSpo, targetInstanceObject, this.patternMatcher.getVariablesScope(), this.patternMatcher);

					return ECheckResult.FAIL;
				}
			}
		}
		else
		{
			return ECheckResult.UNKNOWN;
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public boolean match(final MatchState matchState) throws SDMException
	{
		final AbstractStoryPatternObject sourceSpo = this.storyPatternLink.getSource();
		final AbstractStoryPatternObject targetSpo = this.storyPatternLink.getTarget();

		assert this.patternMatcher.isBound(sourceSpo);
		assert !this.patternMatcher.isBound(targetSpo);

		final EObject sourceInstanceObject = (EObject) this.patternMatcher.getInstanceObject(sourceSpo);

		/*
		 * Evaluate link expression
		 */
		final Variable<EClassifier> result = this.patternMatcher.getExpressionInterpreterManager().evaluateExpression(
				this.storyPatternLink.getExpression(), sourceSpo.getType(), sourceInstanceObject, this.patternMatcher.getVariablesScope());

		assert result != null;

		this.patternMatcher.getNotificationEmitter().traversingLink(this.storyPatternLink, sourceSpo, sourceInstanceObject, targetSpo,
				this.patternMatcher.getVariablesScope(), this.patternMatcher);

		if (result.getValue() instanceof Collection<?>)
		{
			for (final Object targetObject : (Collection<Object>) result.getValue())
			{
				if (this.patternMatcher.matchStoryPatternObject(targetSpo, targetObject))
				{
					return true;
				}
			}
		}
		else
		{
			if (result.getValue() != null && this.patternMatcher.matchStoryPatternObject(targetSpo, result.getValue()))
			{
				return true;
			}
		}

		this.patternMatcher.getNotificationEmitter().storyPatternObjectNotBound(targetSpo, this.patternMatcher.getVariablesScope(),
				this.patternMatcher);

		return false;
	}

	@Override
	public int calculateMatchingCost()
	{
		assert !(this.patternMatcher.isBound(this.storyPatternLink.getSource()) && this.patternMatcher.isBound(this.storyPatternLink
				.getTarget()));

		if (this.patternMatcher.isBound(this.storyPatternLink.getSource()))
		{
			/*
			 * TODO Perform a thorough analysis of the expression to give a
			 * useful estimate.
			 */
			return 42 + this.getNumberOfIncomingUnidirectionalLinks(this.storyPatternLink.getTarget());
		}
		else
		{
			return PatternPart.MATCHING_NOT_POSSIBLE;
		}
	}

	@Override
	public EMatchType doGetMatchType()
	{
		switch (this.storyPatternLink.getModifier())
		{
			case NONE:
				switch (this.storyPatternLink.getMatchType())
				{
					case DEFAULT:
						return EMatchType.MANDATORY;
					case OPTIONAL:
						return EMatchType.OPTIONAL;
				}
			case CREATE:
				throw new UnsupportedOperationException();
			case DESTROY:
				switch (this.storyPatternLink.getMatchType())
				{
					case DEFAULT:
						return EMatchType.MANDATORY;
					case OPTIONAL:
						return EMatchType.OPTIONAL;
				}
			default:
				throw new UnsupportedOperationException();
		}
	}

	@Override
	public MatchState createMatchState()
	{
		/*
		 * We do not need a match state for expression links.
		 */
		return null;
	}
}
