package de.mdelab.mlsdm.interpreter.patternMatcher.patternPartBased;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EStructuralFeature;

import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.ActivityEdge;
import de.mdelab.mlsdm.ActivityNode;
import de.mdelab.mlsdm.interpreter.facade.MLSDMMetamodelFacadeFactory;
import de.mdelab.mlstorypatterns.AbstractStoryPatternLink;
import de.mdelab.mlstorypatterns.AbstractStoryPatternObject;
import de.mdelab.mlstorypatterns.ContainmentLink;
import de.mdelab.mlstorypatterns.ExpressionLink;
import de.mdelab.mlstorypatterns.MapEntryLink;
import de.mdelab.mlstorypatterns.MlstorypatternsPackage;
import de.mdelab.mlstorypatterns.StoryPattern;
import de.mdelab.mlstorypatterns.StoryPatternLink;
import de.mdelab.sdm.interpreter.core.SDMException;
import de.mdelab.sdm.interpreter.core.expressions.ExpressionInterpreterManager;
import de.mdelab.sdm.interpreter.core.facade.MetamodelFacadeFactory;
import de.mdelab.sdm.interpreter.core.notifications.NotificationEmitter;
import de.mdelab.sdm.interpreter.core.patternmatcher.MatchingStrategy;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.DefaultMatchingStrategyWithLog;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.PatternPart;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.PatternPartBasedMatcher;
import de.mdelab.sdm.interpreter.core.variables.VariablesScope;

/**
 * Implementation of the pattern part based matcher for the MLSDM metamodel.
 * 
 * @author Stephan Hildebrandt
 * 
 */
public class MLSDMPatternPartBasedMatcher
		extends
		PatternPartBasedMatcher<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> {

	public MLSDMPatternPartBasedMatcher(
			final StoryPattern storyPattern,
			final VariablesScope<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> variablesScope,
			final MetamodelFacadeFactory<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> p_facadeFactory,
			final ExpressionInterpreterManager<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> expressionInterpreterManager,
			final NotificationEmitter<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> notificationEmitter)
	throws SDMException	{
		this(	storyPattern,
				variablesScope,
				new DefaultMatchingStrategyWithLog<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>( MLSDMMetamodelFacadeFactory.INSTANCE ), 
				p_facadeFactory, //MLSDMMetamodelFacadeFactory.INSTANCE, 
				expressionInterpreterManager,
				notificationEmitter );
	}

//	public MLSDMPatternPartBasedMatcher(
//			final StoryPattern storyPattern,
//			final VariablesScope<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> variablesScope,
//			final MatchingStrategy<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> matchingStrategy,
//			final ExpressionInterpreterManager<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> expressionInterpreterManager,
//			final NotificationEmitter<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> notificationEmitter)
//			throws SDMException
//	{
//		this(	storyPattern,
//				variablesScope,
//				matchingStrategy, 
//				MLSDMMetamodelFacadeFactory.INSTANCE, 
//				expressionInterpreterManager,
//				notificationEmitter );
//	}

	public MLSDMPatternPartBasedMatcher(
			final StoryPattern storyPattern,
			final VariablesScope<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> variablesScope,
			final MatchingStrategy<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> matchingStrategy,
			final MetamodelFacadeFactory<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> facadeFactory,
			final ExpressionInterpreterManager<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> expressionInterpreterManager,
			final NotificationEmitter<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> notificationEmitter)
			throws SDMException
	{
		super(storyPattern, variablesScope, matchingStrategy, facadeFactory, expressionInterpreterManager, notificationEmitter);
	}

	@Override
	protected Collection<PatternPart<AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>> createPatternParts()
	{
		final List<AbstractStoryPatternLink> storyPatternLinks = new LinkedList<AbstractStoryPatternLink>(this.getStoryPattern()
				.getStoryPatternLinks());

		final Collection<PatternPart<AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>> patternParts = new HashSet<PatternPart<AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>>();

		final Set<AbstractStoryPatternObject> mapEntryValueTargets = new HashSet<AbstractStoryPatternObject>();

		/*
		 * Create NAC pattern parts for NACs
		 */
		for (final StoryPattern nacPattern : this.getStoryPattern().getNegativeApplicationConditions())
		{
			patternParts.add(new MLSDMNACPatternPart(this, nacPattern));
		}

		/*
		 * Split the story pattern into pattern parts. A pattern part consists
		 * either of a single isolated story pattern object or a link with its
		 * connected objects.
		 */
		while (!storyPatternLinks.isEmpty())
		{
			final AbstractStoryPatternLink link = storyPatternLinks.remove(0);

			final EClass linkEClass = link.eClass();

			MLSDMPatternPart<AbstractStoryPatternObject, ? extends AbstractStoryPatternLink> part = null;

			if (linkEClass == MlstorypatternsPackage.Literals.STORY_PATTERN_LINK)
			{
				part = new MLSDMStoryPatternLinkPatternPart(this, (StoryPatternLink) link);
			}
			else if (linkEClass == MlstorypatternsPackage.Literals.CONTAINMENT_LINK)
			{
				part = new MLSDMContainmentLinkPatternPart(this, (ContainmentLink) link);
			}
			else if (linkEClass == MlstorypatternsPackage.Literals.EXPRESSION_LINK)
			{
				part = new MLSDMExpressionLinkPatternPart(this, (ExpressionLink) link);
			}
			else if (linkEClass == MlstorypatternsPackage.Literals.MAP_ENTRY_LINK)
			{
				final MapEntryLink mapLink = (MapEntryLink) link;

				part = new MLSDMMapEntryLinkPatternPart(this, mapLink);

				if (mapLink.getValueTarget() != null)
				{
					mapEntryValueTargets.add(mapLink.getValueTarget());
				}
			}
			else
			{
				throw new UnsupportedOperationException();
			}

			assert part != null;

			patternParts.add(part);
		}

		/*
		 * Create pattern parts for objects that are not connected to any links.
		 */
		for (final AbstractStoryPatternObject spo : this.getStoryPattern().getStoryPatternObjects())
		{
			if (spo.getIncomingLinks().isEmpty() && spo.getOutgoingLinks().isEmpty() && !mapEntryValueTargets.contains(spo))
			{
				final MLSDMStoryPatternObjectOnlyPatternPart part = new MLSDMStoryPatternObjectOnlyPatternPart(this, spo);

				patternParts.add(part);
			}
		}

		return patternParts;
	}
}
